@@ -1,9 +1,30 @@
+#######################################################################
+# arulesViz - Visualizing Association Rules and Frequent Itemsets
+# Copyrigth (C) 2011 Michael Hahsler and Sudheer Chelluboina
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
+
 ### default with alpha
 .nodeColors <- function(alpha=NULL) {
     if(is.null(alpha)) alpha <- 1
     c(rgb(.4,.8,.4, alpha), rgb(.6,.6,.8, alpha))
 }
 
+.font.family <- "sans"
+
 gray_hcl <- function(level, alpha=NULL) hcl(c=0, l=level*100, 
 	alpha=if(!is.null(alpha)) alpha else 1)
 
@@ -25,15 +46,24 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 		    measureLabels = FALSE,
 		    precision = 3,
 		    type = "itemsets",
-		    layout = layout.fruchterman.reingold,
+		    layout = NULL,
+		    layoutParams = list(),
 		    arrowSize = .5,
 		    interactive = FALSE,
+		    engine = "igraph",
 		    plot = TRUE
 		    ), control)
 
     opar <- par(mar = c(0,0.1,4,0.1))
     on.exit(par(opar))
 
+
+    ### default layout for igraph
+    if(control$interactive  || (control$engine == "igraph" 
+		    && is.null(control$layout))) {
+	control$layout <- layout.fruchterman.reingold
+    }
+
     if(control$type=="bipartite") {
 	## make bipartite graph
 	leftHandSide <- as.factor(labels(lhs(rules))$elements)
@@ -112,10 +142,11 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 		rep(2, length(levels(rightHandSide))))]
 	
 	if(!control$interactive) {
-	    
-	    plot(g, layout=control$layout, 
-		    vertex.label.family="Helvetica",
-		    edge.label.family="Helvetica",
+	   
+	    plot(g, ...,
+		    layout=control$layout(g, params=control$layoutParams),
+		    vertex.label.family=.font.family,
+		    edge.label.family=.font.family,
 		    vertex.shape=v.shape, 
 		    vertex.label=v,
 		    vertex.label.cex=control$cex,
@@ -127,12 +158,13 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 		    edge.label.cex=control$cex*.6,
 		    edge.color=e.color,
 		    edge.arrow.size=control$arrowSize,
-		    main=control$main,...
+		    main=control$main
 		    )
 
 	    mtext(legend, adj=1,cex=control$cex*.8)
 	}else {
-	    tkplot(g, layout=control$layout, 
+	    tkplot(g, ...,
+		    layout=control$layout(g, params=control$layoutParams),
 		    #vertex.shape=v.shape, 
 		    vertex.label=v,
 		    #vertex.label.cex=.7,
@@ -221,29 +253,109 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 	#v.color <- NA   
 
 	if(!control$interactive) {
+	    if(control$engine=="graphviz") {
+		if(!require("Rgraphviz")) stop ("Package Rgraphviz needed!")
 
-	    plot(g, layout=control$layout, 
-		    vertex.label.family="Helvetica",
-		    edge.label.family="Helvetica",
-		    vertex.shape=v.shape, 
-		    vertex.label=v,
-		    vertex.label.cex=control$cex,
-		    vertex.label.color="black",
-		    #vertex.color = v.color,
-		    #vertex.size=v.size,
-		    edge.width=e.width,
-		    edge.label=e.label,
-		    edge.label.cex=control$cex*.6,
-		    edge.color=e.color,
-		    edge.arrow.size=control$arrowSize,
-		    main=control$main,...
-		    )
+		gNEL <- igraph.to.graphNEL(g)
+		
+		if(is.null(control$layout)) control$layout <- "neato"
+		att <-  getDefaultAttrs(layoutType = control$layout)
+		#att$node$width <- .75
+		#att$node$fillcolor <- "#e0e0e0"
+		#att$node$shape <- "ellipse"
+		#att$node$fixedsize <- FALSE
+		#att$node$fontsize <- 8.0
+		#att$edge$weight <- 4.0
+		att$edge$len <- 2.5 # neato
+		att$edge$color <- black_hcl(control$alpha)
 
-	    mtext(legend, adj=1,cex=control$cex*.8)
-	
+		## edges
+		eAttrs <- list()
+		if(control$measureLabels) {
+		    edgeLabels <- paste(paste(round(m, control$precision), 
+				    "/", round(s, control$precision)))
+		    edgeLabels <- edgeLabels[setdiff(seq(along = edgeLabels),
+			    removedEdges(gNEL))]
+		    names(edgeLabels) <- edgeNames(gNEL)
+		    eAttrs$label <- edgeLabels
+		    #labelfontsize <- rep("1", length(edgeNames(graph)))
+		    #names(labelfontsize) <- edgeNames(graph)
+		    #eAttrs$labelfontsize <- labelfontsize
+		}
+
+		## use gray value to code for measure
+		#color <- paste("grey", floor(map(m, c(80,1))), sep='')
+		if(!is.na(measure)) {
+		    lwd <- map(sapply(edgeData(gNEL), 
+				    FUN = function(i) i[[measure]]),
+			    c(2,5))
+		    names(lwd) <- edgeNames(gNEL)
+		    eAttrs$lwd <- lwd
+		}
+
+		if(!is.na(shading)) {
+		    color <- gray_hcl(map(sapply(edgeData(gNEL), 
+				    FUN = function(i) i[[shading]]),
+			    c(0.9,0.1)), alpha=control$alpha)
+		    names(color) <- edgeNames(gNEL)
+		    eAttrs$color <- color
+		}
+
+
+		if(control$itemLabels) {
+		    nAttrs <- makeNodeAttrs(gNEL, 
+			    fillcolor = control$nodeColors[1], 
+			    label = levels(itemsetLabels)[as.integer(nodes(gNEL))],
+			    shape = "box", width=0.75, fixedsize=FALSE)
+		} else {
+		    nAttrs <- makeNodeAttrs(gNEL, 
+			    fillcolor = control$nodeColors[1],
+			    shape = "circle")
+
+		   # writeLines("Itemsets")
+		   #  print(levels(itemsetLabels))
+		}
+
+		## plot whines about zero length arrows
+		suppressWarnings(pl <- plot(gNEL, control$layout, 
+				edgeAttrs = eAttrs,
+				nodeAttrs = nAttrs,
+				attrs = att,
+				recipEdges="distinct", 
+				main=control$main))
+
+		## legend
+		text(pl@boundBox@upRight@x, pl@boundBox@upRight@y*1.05,
+				    legend, pos=2,cex=.8, xpd=NA)
+		
+		return(invisible(pl))
+
+	    }else{ ## igraph
+		plot(g, ..., 
+			layout=control$layout(g, params=control$layoutParams),
+			vertex.label.family=.font.family,
+			edge.label.family=.font.family,
+			vertex.shape=v.shape, 
+			vertex.label=v,
+			vertex.label.cex=control$cex,
+			vertex.label.color="black",
+			#vertex.color = v.color,
+			#vertex.size=v.size,
+			edge.width=e.width,
+			edge.label=e.label,
+			edge.label.cex=control$cex*.6,
+			edge.color=e.color,
+			edge.arrow.size=control$arrowSize,
+			main=control$main
+			)
+
+		mtext(legend, adj=1,cex=control$cex*.8)
+
+	    }
 	}else {
 	    
-	    tkplot(g, layout=control$layout, 
+	    tkplot(g, ..., 
+		    layout=control$layout(g, params=control$layoutParams),
 		    #vertex.shape=v.shape, 
 		    vertex.label=v,
 		    #vertex.label.cex=.7,
@@ -261,6 +373,7 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 
     } else if(control$type=="items") {
 	
+	### find used items
 	itemNodes <- which(itemFrequency(items(generatingItemsets(rules)), 
 			type="absolute") >0)
 
@@ -277,7 +390,6 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 
 	type <- c(rep(1, length(itemNodes)), rep(2, length(ruleNodes)))
 	
-	#nodeLabels <- c(itemLabels(rules)[itemNodes], ruleNodes)
 	nodeLabels <- c(itemLabels(rules)[itemNodes], rep("", length(ruleNodes)))
 
 	e.list <- cbind(c(from_lhs, from_rhs), c(to_lhs, to_rhs))
@@ -288,14 +400,15 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 
 	g <- graph.data.frame(e.list, directed=TRUE, vertices=v.labels)
 	
-	if(!control$plot) return(g)
-	
+	## add quality measures
 	for(m in names(quality(rules))) {
 	    g <- set.vertex.attribute(g, m, which(type==2)-1, 
 		    quality(rules)[[m]])
 	}
 
+	if(!control$plot) return(g)
 	
+	## create representation	
 	v <- v.labels[,1]
 	v[type==2] <- ""
 	v.shape <- c("rectangle","circle")[type]
@@ -349,28 +462,86 @@ graph_arules <- function(rules, measure = "support", shading = "lift",
 		sep ='')
 
 
+	
+	
 	if(!control$interactive) {
+	    if(control$engine=="graphviz") {
+		if(!require("Rgraphviz")) stop ("Package Rgraphviz needed!")
+		
+		gNEL <- igraph.to.graphNEL(g)
 
-	    plot(g, layout=control$layout, 
-		    vertex.label.family="Helvetica",
-		    edge.label.family="Helvetica",
-		    vertex.shape=v.shape, 
-		    vertex.label=v,
-		    vertex.label.cex=control$cex,
-		    vertex.label.color="black",
-		    vertex.color = v.color,
-		    vertex.size=v.size,
-		    edge.width=e.width,
-		    #edge.label=e.label,
-		    edge.label.cex=control$cex*.6,
-		    edge.color=e.color,
-		    edge.arrow.size=control$arrowSize,
-		    main=control$main, ...
-		    )
+		if(is.null(control$layout)) control$layout <- "dot"
+		att <-  getDefaultAttrs(layoutType = control$layout)
+		att$edge$color <- black_hcl(control$alpha)
+		att$edge$len <- 2.0	# neato
+		att$graph$rankdir <- "LR" # dot
+		att$graph$ranksep <- .75 #dot
+		att$graph$mclimit <- 1 #dot
+	
+		## TODO: plot edge labels
 
-	    mtext(legend, adj=1,cex=control$cex*.8)
+		if(control$itemLabels) {
+		    nAttrs <- makeNodeAttrs(gNEL, 
+			    fillcolor = c(rep(control$nodeColors[1], 
+					    length(itemNodes)), 
+				    gray_hcl(map(s, c(0.9,0.1)), 
+					    alpha=control$alpha)), 
+			    label = nodeLabels,
+			    shape = c("box", "circle")[type], 
+			    width=c(rep(.75, length(itemNodes)), 
+				    map(m, c(0.5,1.2))), 
+			    fixedsize=FALSE)
+		} else {
+		    nAttrs <- makeNodeAttrs(gNEL, 
+			    fillcolor = c(rep(control$nodeColors[1],
+					    length(itemNodes)),
+				    gray_hcl(map(s, c(0.9,0.1)), 
+					    alpha=control$alpha)),
+			    shape = c("box", "circle")[type],
+			    width=c(rep(.75, length(itemNodes)),
+				    map(m, c(0.5,1.2))),
+			    fixedsize=FALSE
+			    )
+		    #writeLines("Itemsets")
+		    #print(levels(itemsetLabels))
+		}
+		
+		## plot whines about zero length arrows
+		suppressWarnings(pl <- plot(gNEL, control$layout, 
+				nodeAttrs = nAttrs,
+				attrs = att,
+				recipEdges="distinct", 
+				main=control$main))
+		
+		text(pl@boundBox@upRight@x, pl@boundBox@upRight@y*1.05,
+				    legend, pos=2,cex=.8, xpd=NA)
+		
+		return(invisible(pl))
+	    
+	    }else{ ## igraph
+		plot(g, ...,
+			layout=control$layout(g, params=control$layoutParams), 
+			vertex.label.family="Helvetica",
+			edge.label.family="Helvetica",
+			vertex.shape=v.shape, 
+			vertex.label=v,
+			vertex.label.cex=control$cex,
+			vertex.label.color="black",
+			vertex.color = v.color,
+			vertex.size=v.size,
+			edge.width=e.width,
+			#edge.label=e.label,
+			edge.label.cex=control$cex*.6,
+			edge.color=e.color,
+			edge.arrow.size=control$arrowSize,
+			main=control$main
+			)
+
+		mtext(legend, adj=1,cex=control$cex*.8)
+	    }
 	}else {
-	    tkplot(g, layout=control$layout, 
+	    tkplot(g, ..., 
+		    layout=control$layout(g, params=control$layoutParams),
 		    #vertex.shape=v.shape, 
 		    vertex.label=v,
 		    #vertex.label.cex=.7,
@@ -396,7 +567,7 @@ graph_arules_is <- function(itemsets, measure = "support", shading = NULL,
 
     control <- .get_parameters(list(
 		    main =paste("Graph for", length(itemsets), "itemsets"),
-		    nodeColors = gray_hcl(.8, 
+		    nodeColors = .nodeColors( 
 			    if(!is.null(control$alpha)) control$alpha else .8),
 		    edgeColors = gray_hcl(.3, 
 			    if(!is.null(control$alpha)) control$alpha else .8),
@@ -405,7 +576,9 @@ graph_arules_is <- function(itemsets, measure = "support", shading = NULL,
 		    cex = 1,
 		    precision = 3,
 		    type = "items",
-		    layout = layout.fruchterman.reingold,
+		    layout = NULL,
+		    layoutParams = list(),
+		    engine = "igraph",
 		    alpha = .8,
 		    interactive = FALSE,
 		    plot = TRUE
@@ -413,6 +586,12 @@ graph_arules_is <- function(itemsets, measure = "support", shading = NULL,
 
     opar <- par(mar = c(0,0.1,4,0.1))
     on.exit(par(opar))
+    
+    ### default layout for igraph
+    if(control$interactive  || (control$engine == "igraph" 
+		    && is.null(control$layout))) {
+	control$layout <- layout.fruchterman.reingold
+    }
 
 
     if(control$type=="items") {
@@ -442,7 +621,7 @@ graph_arules_is <- function(itemsets, measure = "support", shading = NULL,
 
 
 	v.shape <- c("circle","none")[type]
-	v.color <- control$nodeColors
+	v.color <- control$nodeColors[type]
 	
 	e.width <- 1
 	e.color <- control$edgeColors
@@ -466,43 +645,88 @@ graph_arules_is <- function(itemsets, measure = "support", shading = NULL,
 		sep='')
 
 	if(!control$interactive) {
-	
-	    plot(g, layout=control$layout, 
-		vertex.label.family="Helvetica",
-		edge.label.family="Helvetica",
-		vertex.shape=v.shape, 
-		vertex.label=v,
-		vertex.label.cex=control$cex,
-		vertex.label.color="black",
-		vertex.color = v.color,
-		vertex.size=v.size,
-		edge.width=e.width,
-		edge.label=e.label,
-		edge.label.cex=control$cex*.6,
-		edge.color=e.color,
-		main=control$main,
-		...
-		)
+	    if(control$engine=="graphviz") {
+		if(!require("Rgraphviz")) stop ("Package Rgraphviz needed!")
+		
+		gNEL <- igraph.to.graphNEL(g)
+
+		if(is.null(control$layout)) control$layout <- "dot"
+		
+		nAttrs <- makeNodeAttrs(gNEL, 
+			fillcolor = control$nodeColors[type], 
+			#    label = nodeLabels,
+			shape = c("circle","box")[type], 
+			width=c(c(map(m, c(.1,2))), rep(.75, length(items))), 
+			fixedsize=c(TRUE, FALSE)[type])
+
+		sgL <- list(list(graph = subGraph(as.character(sets), gNEL), 
+				cluster = TRUE,
+				attrs = c(rank = "sink")))
+		att <- list(graph = list(
+				rankdir = "LR", #dot
+				rank = "min", #dot
+				ranksep = .75, #dot
+				mclimit = 1 #dot
+				),
+			edge=list(
+				len = 2, #neato
+				color = rgb(0,0,0,control$alpha)
+				))
+
+		## TODO: plot edge/item labels
+
+		## plot whines about zero length arrows
+		suppressWarnings(pl <- plot(gNEL, control$layout, 
+				subGList = sgL,
+				nodeAttrs = nAttrs, 
+				attrs = att,
+				recipEdges="distinct", 
+				main=control$main))
+
+		text(pl@boundBox@upRight@x, pl@boundBox@upRight@y*1.05,
+			legend, pos=2,cex=.8, xpd=NA)
+
+		return(invisible(pl))
+
+	    }else{ ## igraph
+
+		plot(g, ...,
+			layout=control$layout(g, params=control$layoutParams),
+			vertex.label.family=.font.family,
+			edge.label.family=.font.family,
+			vertex.shape=v.shape, 
+			vertex.label=v,
+			vertex.label.cex=control$cex,
+			vertex.label.color="black",
+			vertex.color = v.color,
+			vertex.size=v.size,
+			edge.width=e.width,
+			edge.label=e.label,
+			edge.label.cex=control$cex*.6,
+			main=control$main
+			)
+
+		mtext(legend, adj=1,cex=control$cex*.8)
+	    }
+	}else{
+
+	    tkplot(g, ...,
+		    layout=control$layout(g, params=control$layoutParams),
+		    vertex.shape=v.shape, 
+		    vertex.label=v,
+		    #vertex.label.cex=.7,
+		    #vertex.label.color="black",
+		    vertex.color = v.color,
+		    vertex.size=v.size,
+		    #edge.width=e.width,
+		    edge.label=e.label,
+		    #edge.label.cex=.5,
+		    edge.color=e.color,
+		    #main=control$main
+		    )
+
+	}
 
-	mtext(legend, adj=1,cex=control$cex*.8)
-    }else{
-    
-	    tkplot(g, layout=control$layout, 
-		vertex.shape=v.shape, 
-		vertex.label=v,
-		#vertex.label.cex=.7,
-		#vertex.label.color="black",
-		vertex.color = v.color,
-		vertex.size=v.size,
-		#edge.width=e.width,
-		edge.label=e.label,
-		#edge.label.cex=.5,
-		edge.color=e.color,
-		#main=control$main
-		)
-    
-    }
-	
 	return(invisible(g))
     }
 }
